"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Result = void 0;
var result_status_1 = require("./result-status");
var Result = /** @class */ (function () {
    function Result(value, status, error, errorMessage, metaData) {
        if (error === void 0) { error = null; }
        if (errorMessage === void 0) { errorMessage = null; }
        if (metaData === void 0) { metaData = null; }
        this.hasError = status !== result_status_1.ResultStatus.Success;
        this.value = value;
        this.isSuccess = status === result_status_1.ResultStatus.Success;
        this.isFailure = status === result_status_1.ResultStatus.Failure;
        this.isConflict = status === result_status_1.ResultStatus.Conflict;
        this.isInvalid = status === result_status_1.ResultStatus.Invalid;
        this.isForbidden = status === result_status_1.ResultStatus.Forbidden;
        this.isUnauthorized = status === result_status_1.ResultStatus.Unauthorized;
        this.isUnsupported = status === result_status_1.ResultStatus.Unsupported;
        this.isNotFound = status === result_status_1.ResultStatus.NotFound;
        this.status = status;
        this.error = error;
        this.errorMessage = errorMessage;
        this.metaData = metaData;
    }
    Result.conflict = function (errorMessage, error, metaData) {
        return new Result(null, result_status_1.ResultStatus.Conflict, error, errorMessage, metaData);
    };
    Result.failure = function (errorMessage, error, metaData) {
        return new Result(null, result_status_1.ResultStatus.Failure, error, errorMessage, metaData);
    };
    Result.forbidden = function (errorMessage, error, metaData) {
        return new Result(null, result_status_1.ResultStatus.Forbidden, error, errorMessage, metaData);
    };
    Result.invalid = function (errorMessage, error, metaData) {
        return new Result(null, result_status_1.ResultStatus.Invalid, error, errorMessage, metaData);
    };
    Result.notFound = function (errorMessage, error, metaData) {
        return new Result(null, result_status_1.ResultStatus.NotFound, error, errorMessage, metaData);
    };
    Result.success = function (value) {
        return new Result(value, result_status_1.ResultStatus.Success);
    };
    Result.unauthorized = function (errorMessage, error, metaData) {
        return new Result(null, result_status_1.ResultStatus.Unauthorized, error, errorMessage, metaData);
    };
    Result.unsupported = function (errorMessage, error, metaData) {
        return new Result(null, result_status_1.ResultStatus.Unsupported, error, errorMessage, metaData);
    };
    Result.prototype.addOrUpdateMetadata = function (key, value) {
        var _a;
        if (!key)
            throw new Error("Value cannot be null or whitespace.");
        var newMetaData = __assign(__assign({}, this.metaData), (_a = {}, _a[key] = value, _a));
        return new Result(this.value, this.status, this.error, this.errorMessage, newMetaData);
    };
    Result.prototype.map = function (mapper) {
        return this.isSuccess
            ? Result.success(mapper(this.value))
            : Result.failure(this.errorMessage || undefined, this.error || undefined, this.metaData || undefined);
    };
    Result.prototype.recover = function (recovery) {
        return this.isSuccess ? this : recovery(this.value);
    };
    Result.prototype.match = function (success, failure) {
        return this.isSuccess
            ? success(this.value)
            : failure(this.errorMessage, this.error, this.metaData);
    };
    Result.prototype.unwrapOrNull = function () {
        return this.isSuccess ? this.value : null;
    };
    Result.prototype.unwrapOrDefault = function (defaultValue) {
        return this.isSuccess ? this.value : defaultValue;
    };
    Result.prototype.unwrap = function () {
        if (this.isSuccess) {
            return this.value;
        }
        throw new Error("Cannot unwrap an failed result.");
    };
    Result.prototype.unwrapOrThrow = function (errorProvider) {
        if (this.isSuccess) {
            return this.value;
        }
        throw errorProvider();
    };
    return Result;
}());
exports.Result = Result;
//# sourceMappingURL=result.js.map