"use strict";
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.toCss = void 0;
/**
 * Converts a CSS object to a CSS string.
 * @param {CssProperties | CssNestedProperties} cssObj - The CSS object to convert.
 * @returns {string} The CSS string.
 * @throws {TypeError} If the input is not a valid CSS object.
 */
function toCss(cssObj) {
    return convertToCss(cssObj, []);
}
exports.toCss = toCss;
/**
 * Converts a CSS object to a CSS string recursively.
 * @param {CssProperties | CssNestedProperties} cssObj - The CSS object to convert.
 * @param {string[]} parentSelectors - The array of parent selectors.
 * @returns {string} The CSS string.
 * @throws {TypeError} If the input is not a valid CSS object.
 */
function convertToCss(cssObj, parentSelectors) {
    var e_1, _a;
    if (parentSelectors === void 0) { parentSelectors = []; }
    if (!cssObj || typeof cssObj !== "object" || Array.isArray(cssObj)) {
        throw new TypeError("Expected an argument of type object, but got ".concat(typeof cssObj));
    }
    var lines = [];
    var atRules = [];
    try {
        for (var _b = __values(Object.entries(cssObj)), _c = _b.next(); !_c.done; _c = _b.next()) {
            var _d = __read(_c.value, 2), key = _d[0], value = _d[1];
            var selectors = __spreadArray([], __read(parentSelectors), false);
            if (key.startsWith("--")) {
                if (typeof value === "string" || typeof value === "number") {
                    lines.push("".concat(key, ": ").concat(value, ";"));
                }
                else {
                    throw new TypeError("Expected a value of type string or number for CSS variable, but got ".concat(typeof value));
                }
                continue;
            }
            if (key.startsWith("@")) {
                if (typeof value === "object" &&
                    value !== null &&
                    !Array.isArray(value)) {
                    atRules.push("".concat(key, " { ").concat(convertToCss(value, []), " }"));
                }
                else {
                    throw new TypeError("Expected an object for CSS at-rule, but got ".concat(typeof value));
                }
                continue;
            }
            if (key.startsWith("&")) {
                selectors[selectors.length - 1] += key.slice(1);
            }
            else {
                selectors.push(caseConverter(key));
            }
            if (typeof value === "object" &&
                value !== null &&
                !Array.isArray(value)) {
                lines.push("".concat(convertToCss(value, selectors)));
            }
            else if (typeof value === "string" ||
                typeof value === "number" ||
                typeof value === "boolean") {
                lines.push("".concat(selectors.join(" "), ": ").concat(value, ";"));
            }
            else {
                throw new TypeError("Expected a value of type string, number, or boolean, but got ".concat(typeof value));
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return __spreadArray(__spreadArray([], __read(lines), false), __read(atRules), false).join(" ");
    /**
     * Converts a string to kebab-case.
     * @param {string} str - The string to convert.
     * @returns {string} The converted string in kebab-case.
     */
    function caseConverter(str) {
        if (str.length === 0)
            return str;
        var isUppercase = str[0] === str[0].toUpperCase();
        var result = str
            .split(/(?=[A-Z])/)
            .join("-")
            .toLowerCase();
        return isUppercase ? "-".concat(result) : result;
    }
}
//# sourceMappingURL=css-utilities.js.map