/**
 * Converts a string to camel case.
 */
declare function camelCase(str: string): string;
/**
 * Capitalizes the first character of a string.
 */
declare function capitalize(str: string): string;
/**
 * Removes diacritical marks from a string.
 */
declare function deburr(str: string): string;
/**
 * Checks if a string ends with the given target string.
 */
declare function endsWith(str: string, target: string, position?: number): boolean;
/**
 * Escapes special characters in a string.
 */
declare function escape(str: string): string;
/**
 * Escapes special characters in a regular expression string.
 */
declare function escapeRegExp(str: string): string;
/**
 * Converts a string to kebab case.
 */
declare function kebabCase(str: string): string;
/**
 * Converts a string to lower case.
 */
declare function lowerCase(str: string): string;
/**
 * Converts the first character of a string to lower case.
 */
declare function lowerFirst(str: string): string;
/**
 * Pads a string on both sides with characters until it reaches the given length.
 */
declare function pad(str: string, length: number, chars?: string): string;
/**
 * Pads a string on the end with characters until it reaches the given length.
 */
declare function padEnd(str: string, length: number, chars?: string): string;
/**
 * Pads a string on the start with characters until it reaches the given length.
 */
declare function padStart(str: string, length: number, chars?: string): string;
/**
 * Parses a string to an integer of the specified radix.
 */
declare function parseInt(str: string, radix?: number): number;
/**
 * Repeats a string `n` times.
 */
declare function repeat(str: string, n: number): string;
/**
 * Replaces all occurrences of a pattern in a string with a replacement.
 */
declare function replace(str: string, pattern: RegExp | string, replacement: string): string;
/**
 * Converts a string to snake case.
 */
declare function snakeCase(str: string): string;
/**
 * Splits a string into an array of words.
 */
declare function split(str: string, separator: RegExp | string, limit?: number): string[];
/**
 * Converts a string to start case (capitalizes the first character of each word).
 */
declare function startCase(str: string): string;
/**
 * Checks if a string starts with the given target string.
 */
declare function startsWith(str: string, target: string, position?: number): boolean;
/**
 * Evaluates a template string with interpolated placeholders.
 */
declare function template(str: string, placeholders: Record<string, unknown>): string;
/**
 * Converts a string to lower case.
 */
declare function toLower(str: string): string;
/**
 * Converts a string to upper case.
 */
declare function toUpper(str: string): string;
/**
 * Removes leading and trailing whitespace from a string.
 */
declare function trim(str: string): string;
/**
 * Removes trailing whitespace from a string.
 */
declare function trimEnd(str: string): string;
/**
 * Removes leading whitespace from a string.
 */
declare function trimStart(str: string): string;
/**
 * Truncates a string to a specified length.
 */
declare function truncate(str: string, options?: {
    length: number;
    omission?: string;
    separator?: RegExp | string;
}): string;
/**
 * Unescapes special characters in an HTML string.
 */
declare function unescape(str: string): string;
/**
 * Converts a string to upper case.
 */
declare function upperCase(str: string): string;
/**
 * Converts the first character of a string to upper case.
 */
declare function upperFirst(str: string): string;
/**
 * Splits a string into an array of words.
 */
declare function words(str: string, pattern?: RegExp | string): string[];
/**
 * A function to convert ArrayBuffer to hexadecimal
 */
declare function toHexAsBuffer(buffer: ArrayBuffer): string;
/**
 * A function to convert string to hexadecimal
 */
declare function toHex(s: string): string;
/**
 * UUIDv1 generation
 */
declare function generateUUIDv1(): string;
/**
 * UUIDv4 generation
 */
declare function generateUUIDv4(): string;
/**
 * This function takes a string as input and returns an encoded Uint8Array.
 * It uses the TextEncoder object which is a global object in modern browsers.
 * It provides encoding of string data to bytes using UTF-8 encoding.
 *
 * @param message - The string that should be encoded.
 * @returns The encoded Uint8Array.
 */
declare function encode(message: string): Uint8Array;
/**
 * This function takes a Uint8Array as input and returns a decoded string.
 * It uses the TextDecoder object which is a global object in modern browsers.
 * It provides decoding of text data that has been encoded to bytes.
 *
 * @param data - The Uint8Array that should be decoded.
 * @returns The decoded string.
 */
declare function decode(data: Uint8Array): string;
declare function isNullOrWhiteSpace(str: string): boolean;
declare function isNullOrEmpty(str: string): boolean;
declare function isNotNullOrWhiteSpace(str: string): boolean;
declare function isNotNullOrEmpty(str: string): boolean;
declare function isStringEmptyOrUndefined(tag: string | null | undefined): boolean;
export { camelCase, capitalize, deburr, decode, encode, endsWith, escape, escapeRegExp, generateUUIDv1, generateUUIDv4, isNotNullOrEmpty, isNotNullOrWhiteSpace, isNullOrEmpty, isNullOrWhiteSpace, isStringEmptyOrUndefined, kebabCase, lowerCase, lowerFirst, pad, padEnd, padStart, parseInt, repeat, replace, snakeCase, split, startCase, startsWith, template, toHex, toHexAsBuffer, toLower, toUpper, trim, trimEnd, trimStart, truncate, unescape, upperCase, upperFirst, words, };
//# sourceMappingURL=string-utilities.d.ts.map