import { TypedArray } from "../types";
declare function castArray(value: unknown): unknown[];
declare function clone<T>(value: T): T;
declare function cloneDeep<T>(value: T): T;
declare function cloneDeepWith<T>(value: T, customizer: (value: unknown) => unknown): T;
declare function cloneWith<T>(value: T, customizer: (value: unknown) => unknown): T;
declare function conformsTo<T extends object>(object: T, source: Partial<T>): boolean;
declare function eq(value: unknown, other: unknown): boolean;
declare function gt<T extends number>(value: T, other: T): boolean;
declare function gte<T extends number>(value: T, other: T): boolean;
declare function isArguments(value: unknown): boolean;
declare function isArray(value: unknown): boolean;
declare function isArrayBuffer(value: unknown): boolean;
declare function isArrayLike(value: unknown): boolean;
declare function isArrayLikeObject(value: unknown): boolean;
declare function isBoolean(value: unknown): boolean;
declare function isBuffer(value: unknown): boolean;
declare function isElement(value: unknown): boolean;
declare function isEmpty(value: unknown): boolean;
declare function isEqual(value: unknown, other: unknown): boolean;
declare function isEqualWith(value: unknown, other: unknown, customizer: (v1: unknown, v2: unknown) => boolean): boolean;
declare function isError(value: unknown): boolean;
declare function isFinite(value: unknown): boolean;
declare function isFunction(value: unknown): boolean;
declare function isInteger(value: unknown): boolean;
declare function isLength<T extends number>(value: T): value is T;
declare function isMap(value: unknown): boolean;
declare function isMatch(object: Record<string, unknown>, source: Record<string, unknown>): boolean;
declare function isMatchWith(object: Record<string, unknown>, source: Record<string, unknown>, customizer: (v1: unknown, v2: unknown) => boolean): boolean;
declare function isNaN(value: unknown): boolean;
declare function isNative(value: unknown): boolean;
declare function isNil(value: unknown): boolean;
declare function isNull(value: unknown): boolean;
declare function isObject(value: unknown): boolean;
declare function isObjectLike(value: unknown): boolean;
declare function isPlainObject(value: unknown): boolean;
declare function isRegExp(value: unknown): boolean;
declare function isSafeInteger(value: unknown): boolean;
declare function isSet(value: unknown): boolean;
declare function isString(value: unknown): boolean;
declare function isSymbol(value: unknown): boolean;
declare function isUndefined(value: unknown): boolean;
declare function isWeakMap(value: unknown): boolean;
declare function isWeakSet(value: unknown): boolean;
declare function lt<T extends number>(value: T, other: T): boolean;
declare function lte<T extends number>(value: T, other: T): boolean;
declare function toArray<T>(value: T | T[]): T[];
declare function toFinite(value: unknown): number;
declare function toInteger(value: unknown): number;
declare function toSafeInteger(value: unknown): number;
declare function toString(value: unknown): string;
/**
 * Checks if a value is iterable.
 * @param value The value to check.
 * @returns Returns `true` if the value is iterable, `false` otherwise.
 */
declare function isIterable(value: unknown): value is Iterable<unknown>;
/**
 * Safely converts a JavaScript value to a JSON string, handling circular references.
 * @param value - The value to be converted to a JSON string.
 * @param replacer - Optional. A function that alters the behavior of the stringification process.
 * @param space - Optional. The number of spaces to use for indentation or a string used for indentation.
 * @returns A JSON string representing the input value.
 */
declare function safeStringify(value: unknown, replacer?: (key: string, value: unknown) => unknown, space?: number | string): string;
declare function toNumber(value: unknown): number;
declare function toPlainObject(value: unknown): object;
declare function isNumber(val: unknown): val is number;
declare function isStringArray(val: unknown): val is string[];
declare function isNumberArray(val: unknown): val is number[];
declare function isDate(val: unknown): val is Date;
declare function isPromise(val: unknown): val is Promise<unknown>;
declare function isBigInt(val: unknown): val is bigint;
declare function isNumeric(n: unknown): boolean;
declare function isEmptyObject(obj: object): boolean;
declare function isWindow(obj: unknown): boolean;
declare function type(obj: unknown): string;
declare function isTypedArray(val: unknown): val is TypedArray;
declare function isReadonlyArray<T>(val: unknown): val is ReadonlyArray<T>;
declare function hasSameType<T>(val: unknown, type: T): val is T;
declare function isObjectEmpty(obj: object): boolean;
declare function isLiteral<T extends boolean | number | string>(val: unknown, type: T): val is T;
declare function isUnion<T extends Array<unknown>>(val: unknown, types: T): val is T[number];
declare function isExactUnion<T extends Array<unknown>>(val: unknown, types: T): val is T[number];
declare function isPromiseOfT<T>(obj: Promise<T> | T): obj is Promise<T>;
export { castArray, clone, cloneDeep, cloneDeepWith, cloneWith, conformsTo, eq, gt, gte, hasSameType, isArguments, isArray, isArrayBuffer, isArrayLike, isArrayLikeObject, isBigInt, isBoolean, isBuffer, isDate, isElement, isEmpty, isEmptyObject, isEqual, isEqualWith, isError, isExactUnion, isFinite, isFunction, isInteger, isIterable, isLength, isLiteral, isMap, isMatch, isMatchWith, isNaN, isNative, isNil, isNull, isNumber, isNumberArray, isNumeric, isObject, isObjectEmpty, isObjectLike, isPlainObject, isPromise, isPromiseOfT, isReadonlyArray, isRegExp, isSafeInteger, isSet, isString, isStringArray, isSymbol, isTypedArray, isUndefined, isUnion, isWeakMap, isWeakSet, isWindow, lt, lte, safeStringify, toArray, toFinite, toInteger, toNumber, toPlainObject, toSafeInteger, toString, type, };
//# sourceMappingURL=type-utilities.d.ts.map