type EitherType<L, R> = {
    tag: "Left";
    value: L;
} | {
    tag: "Right";
    value: R;
};
declare class Either {
    static Left<L>(value: L): EitherType<L, never>;
    static Right<R>(value: R): EitherType<never, R>;
    static equals<L, R>(e1: EitherType<L, R>, e2: EitherType<L, R>, equalityFn?: (a: R, b: R) => boolean): boolean;
    static flatMap<L, R, R2>(f: (value: R) => EitherType<L, R2>, e: EitherType<L, R>): EitherType<L, R2>;
    static fold<L, R, T>(onLeft: (value: L) => T, onRight: (value: R) => T, e: EitherType<L, R>): T;
    static getOrElse<L, R>(defaultValue: R, e: EitherType<L, R>): R;
    static isLeft<L, R>(e: EitherType<L, R>): e is {
        tag: "Left";
        value: L;
    };
    static isRight<L, R>(e: EitherType<L, R>): e is {
        tag: "Right";
        value: R;
    };
    static map<L, R, R2>(f: (value: R) => R2, e: EitherType<L, R>): EitherType<L, R2>;
    static orElse<L, R, L2>(f: (value: L) => EitherType<L2, R>, e: EitherType<L, R>): EitherType<L2, R>;
}
export { Either, EitherType };
//# sourceMappingURL=either.d.ts.map