var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
/**
 * `chunk` divides an array into chunks of the specified size
 */
function chunk(array, size) {
    var result = [];
    for (var i = 0; i < array.length; i += size) {
        result.push(array.slice(i, i + size));
    }
    return result;
}
/**
 * `compact` removes all falsy values from an array
 */
function compact(array) {
    return array.filter(function (item) { return Boolean(item); });
}
/**
 * `concat` concatenates multiple arrays
 */
function concat(array) {
    var arrays = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        arrays[_i - 1] = arguments[_i];
    }
    return __spreadArray(__spreadArray([], __read(array), false), __read(arrays.flat()), false);
}
/**
 * `difference` creates an array of array values not included in the other given arrays
 */
function difference(array) {
    var arrays = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        arrays[_i - 1] = arguments[_i];
    }
    var otherValues = new Set(arrays.flat());
    return array.filter(function (value) { return !otherValues.has(value); });
}
/**
 * `drop` creates a slice of array with n elements dropped from the beginning
 */
function drop(array, n) {
    if (n === void 0) { n = 1; }
    return array.slice(n);
}
/**
 * `fill` fills elements of array with value from start up to, but not including, end
 */
function fill(array, value, start, end) {
    if (start === void 0) { start = 0; }
    if (end === void 0) { end = array.length; }
    return array.map(function (v, i) { return (i >= start && i < end ? value : v); });
}
/**
 * `findIndex` gets the index at which the first occurrence of value is found in array
 */
function findIndex(array, predicate, fromIndex) {
    if (fromIndex === void 0) { fromIndex = 0; }
    for (var i = fromIndex; i < array.length; i++) {
        if (predicate(array[i], i, array)) {
            return i;
        }
    }
    return -1;
}
/**
 * `findLastIndex` is like `findIndex` but it iterates over elements of collection from right to left.
 */
function findLastIndex(array, predicate, fromIndex) {
    if (fromIndex === void 0) { fromIndex = array.length - 1; }
    for (var i = fromIndex; i >= 0; i--) {
        if (predicate(array[i], i, array)) {
            return i;
        }
    }
    return -1;
}
/**
 * `first` gets the first element of array.
 */
function first(array) {
    return array[0];
}
/**
 * `flatten` flattens array a single level deep.
 */
function flatten(array) {
    return array.flat();
}
/**
 * `flattenDeep` recursively flattens array.
 */
function flattenDeep(array) {
    return array.reduce(function (acc, val) {
        return Array.isArray(val)
            ? acc.concat(flattenDeep(val))
            : acc.concat(val);
    }, []);
}
/**
 * `flattenDepth` recursively flatten array up to depth times.
 */
function flattenDepth(array, depth) {
    if (depth === void 0) { depth = 1; }
    if (depth < 1)
        return array.slice();
    return array.reduce(function (acc, val) {
        return Array.isArray(val)
            ? acc.concat(flattenDepth(val, depth - 1))
            : acc.concat(val);
    }, []);
}
/**
 * `fromPairs` returns an object composed from key-value pairs.
 */
function fromPairs(pairs) {
    return pairs.reduce(function (acc, _a) {
        var _b;
        var _c = __read(_a, 2), key = _c[0], value = _c[1];
        return (__assign(__assign({}, acc), (_b = {}, _b[key] = value, _b)));
    }, {});
}
/**
 * `head` gets the first element of array.
 */
function head(array) {
    return array[0];
}
/**
 * `indexOf` gets the index at which the first occurrence of value is found in array.
 */
function indexOf(array, value, fromIndex) {
    if (fromIndex === void 0) { fromIndex = 0; }
    for (var i = fromIndex; i < array.length; i++) {
        if (array[i] === value) {
            return i;
        }
    }
    return -1;
}
/**
 * `initial` gets all but the last element of array.
 */
function initial(array) {
    return array.slice(0, array.length - 1);
}
/**
 * `intersection` creates an array of unique values that are included in all given arrays.
 */
function intersection() {
    var arrays = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        arrays[_i] = arguments[_i];
    }
    return arrays.reduce(function (a, b) { return a.filter(function (c) { return b.includes(c); }); });
}
/**
 * `join` converts all elements in array into a string separated by separator.
 */
function join(array, separator) {
    if (separator === void 0) { separator = ","; }
    return array.join(separator);
}
/**
 * `last` gets the last element of array.
 */
function last(array) {
    return array[array.length - 1];
}
/**
 * `lastIndexOf` gets the index at which the last occurrence of value is found in array.
 */
function lastIndexOf(array, value, fromIndex) {
    if (fromIndex === void 0) { fromIndex = array.length - 1; }
    for (var i = fromIndex; i >= 0; i--) {
        if (array[i] === value) {
            return i;
        }
    }
    return -1;
}
/**
 * `nth` gets the element at index n of array. If n is negative, the nth element from the end is returned.
 */
function nth(array, n) {
    if (n === void 0) { n = 0; }
    return n >= 0 ? array[n] : array[array.length + n];
}
/**
 * `pull` removes all given values from array.
 */
function pull(array) {
    var values = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        values[_i - 1] = arguments[_i];
    }
    return array.filter(function (value) { return !values.includes(value); });
}
/**
 * `remove` removes all elements from array that satisfy a condition and returns an array of the removed elements.
 */
function remove(array, predicate) {
    var removed = array.filter(predicate);
    var remaining = array.filter(function (value) { return !predicate(value); });
    array.length = 0;
    array.push.apply(array, __spreadArray([], __read(remaining), false));
    return removed;
}
/**
 * `reverse` reverses array so that the first element becomes the last, the second element becomes the second to last, and so on.
 */
function reverse(array) {
    return array.slice().reverse();
}
/**
 * `slice` creates a slice of array from start up to, but not including, end.
 */
function slice(array, start, end) {
    if (start === void 0) { start = 0; }
    if (end === void 0) { end = array.length; }
    return array.slice(start, end);
}
/**
 * `tail` gets all but the first element of array.
 */
function tail(array) {
    return array.slice(1);
}
/**
 * `take` creates a slice of array with n elements taken from the beginning.
 */
function take(array, n) {
    if (n === void 0) { n = 1; }
    return array.slice(0, n);
}
/**
 * `takeRight` creates a slice of array with n elements taken from the end.
 */
function takeRight(array, n) {
    if (n === void 0) { n = 1; }
    return n === 0 ? [] : array.slice(-n);
}
/**
 * `union` creates an array of unique values, in order, from all given arrays
 */
function union() {
    var arrays = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        arrays[_i] = arguments[_i];
    }
    return Array.from(new Set(arrays.flat()));
}
/**
 * `uniq` creates a duplicate-free version of an array
 */
function uniq(array) {
    return Array.from(new Set(array));
}
/**
 * `unzip` is the opposite of `zip`. It receives an array of grouped elements and creates an array regrouping them to their pre-zip configuration.
 */
function unzip(array) {
    return array[0].map(function (_, i) { return array.map(function (row) { return row[i]; }); });
}
/**
 * `without` creates an array excluding all given values.
 */
function without(array) {
    var values = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        values[_i - 1] = arguments[_i];
    }
    return array.filter(function (value) { return !values.includes(value); });
}
/**
 * `xor` creates an array of unique values that is the symmetric difference of the provided arrays.
 */
function xor() {
    var arrays = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        arrays[_i] = arguments[_i];
    }
    var allValues = new Set(arrays.flat());
    var repeatedValues = new Set(arrays
        .flat()
        .filter(function (value, i, arr) {
        return arr.indexOf(value) !== arr.lastIndexOf(value);
    }));
    return Array.from(allValues).filter(function (value) { return !repeatedValues.has(value); });
}
/**
 * `zip` creates an array of grouped elements, the first of which contains the first elements of the given arrays, the second of which contains the second elements of the given arrays, and so on.
 */
function zip() {
    var arrays = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        arrays[_i] = arguments[_i];
    }
    return arrays[0].map(function (_, i) { return arrays.map(function (array) { return array[i]; }); });
}
/**
 * `zipObject` creates an object composed from arrays of keys and values.
 */
function zipObject(keys, values) {
    return keys.reduce(function (acc, key, i) {
        var _a;
        return (__assign(__assign({}, acc), (_a = {}, _a[key] = values[i], _a)));
    }, {});
}
/**
 * `differenceBy` is like `difference` except that it accepts iteratee which is invoked for each element of array and values to generate the criterion by which they're compared.
 */
function differenceBy(array, values, iteratee) {
    var set = new Set(values.map(iteratee));
    return array.filter(function (value) { return !set.has(iteratee(value)); });
}
/**
 * `pullAll` removes all matching values from array.
 */
function pullAll(array, values) {
    return array.filter(function (value) { return !values.includes(value); });
}
/**
 * `pullAllBy` is like `pullAll` except that it accepts iteratee which is invoked for each element of array and values to generate the criterion by which uniqueness is computed.
 */
function pullAllBy(array, values, iteratee) {
    var set = new Set(values.map(iteratee));
    return array.filter(function (value) { return !set.has(iteratee(value)); });
}
/**
 * `pullAt` removes elements from array corresponding to indexes and returns an array of the removed elements.
 */
function pullAt(array, indexes) {
    var pulled = indexes.map(function (index) { return array[index]; });
    var remaining = array.filter(function (_, i) { return !indexes.includes(i); });
    array.length = 0;
    array.push.apply(array, __spreadArray([], __read(remaining), false));
    return pulled;
}
/**
 * `sortedIndex` uses a binary search to determine the lowest index at which value should be inserted into array in order to maintain its sort order.
 */
function sortedIndex(array, value) {
    var low = 0;
    var high = array.length;
    while (low < high) {
        var mid = (low + high) >>> 1;
        if (array[mid] < value)
            low = mid + 1;
        else
            high = mid;
    }
    return low;
}
/**
 * `unionBy` is like `union` except that it accepts iteratee which is invoked for each element of each arrays to generate the criterion by which uniqueness is computed.
 */
function unionBy(iteratee) {
    var arrays = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        arrays[_i - 1] = arguments[_i];
    }
    var set = new Set();
    return arrays.flat().filter(function (value) {
        var computed = iteratee(value);
        if (!set.has(computed)) {
            set.add(computed);
            return true;
        }
        return false;
    });
}
/**
 * `uniqBy` is like `uniq` except that it accepts iteratee which is invoked for each element in array to generate the criterion by which uniqueness is computed.
 */
function uniqBy(array, iteratee) {
    var set = new Set();
    return array.filter(function (value) {
        var computed = iteratee(value);
        if (!set.has(computed)) {
            set.add(computed);
            return true;
        }
        return false;
    });
}
/**
 * `zipObjectDeep` is like `zipObject` except that it supports property paths.
 */
function zipObjectDeep(props, values) {
    var result = {};
    props.forEach(function (prop, i) {
        var path = prop.split(".");
        path.reduce(function (obj, key, j) {
            if (j === path.length - 1)
                obj[key] = values[i];
            else if (!obj[key])
                obj[key] = {};
            return obj[key];
        }, result);
    });
    return result;
}
/**
 * `sortedIndexOf` uses a binary search to determine the lowest index at which value should be inserted into array in order to maintain its sort order. It accepts a third parameter that determines whether to return the leftmost or rightmost match of the value.
 */
function sortedIndexOf(array, value, isRightmost) {
    if (isRightmost === void 0) { isRightmost = false; }
    var low = 0;
    var high = array.length;
    while (low < high) {
        var mid = Math.floor((low + high) / 2);
        if (array[mid] < value || (isRightmost && array[mid] === value)) {
            low = mid + 1;
        }
        else {
            high = mid;
        }
    }
    return low;
}
/**
 * `sortedLastIndex` is similar to `sortedIndex` except that it returns the highest index at which a value should be inserted into array in order to maintain its sort order.
 */
function sortedLastIndex(array, value) {
    return sortedIndexOf(array, value, true);
}
/**
 * `sortedUniq` is similar to `uniq` except that it’s designed and optimized for sorted arrays.
 */
function sortedUniq(array) {
    return array.filter(function (value, index) { return array.indexOf(value) === index; });
}
/**
 * `takeRightWhile` creates a slice of array with elements taken from the end. Elements are taken until predicate returns falsey.
 */
function takeRightWhile(array, predicate) {
    var i = array.length;
    while (i-- && predicate(array[i])) {
        /* empty */
    }
    return array.slice(i + 1);
}
/**
 * `takeWhile` creates a slice of array with elements taken from the beginning. Elements are taken until predicate returns falsey.
 */
function takeWhile(array, predicate) {
    var i = 0;
    while (i < array.length && predicate(array[i]))
        i++;
    return array.slice(0, i);
}
/**
 * `xorWith` is like `xor`, except that it accepts comparator which is invoked to compare elements of arrays.
 */
function xorWith(arrays, comparator) {
    var allValues = arrays.flat();
    return allValues.filter(function (value, index) {
        return allValues.findIndex(function (otherValue) {
            return comparator(value, otherValue);
        }) === index;
    });
}
/**
 * `zipWith` is like `zip` except that it accepts iteratee to specify how grouped values should be combined.
 */
function zipWith(arrays, iteratee) {
    return zip.apply(void 0, __spreadArray([], __read(arrays), false)).map(function (group) { return iteratee.apply(void 0, __spreadArray([], __read(group), false)); });
}
/**
 * `sortedIndexBy` is like `sortedIndex` except that it accepts iteratee which is invoked for each element in array to generate the sort ranking of the value.
 */
function sortedIndexBy(array, value, iteratee) {
    var low = 0;
    var high = array.length;
    while (low < high) {
        var mid = Math.floor((low + high) / 2);
        if (iteratee(array[mid]) < iteratee(value)) {
            low = mid + 1;
        }
        else {
            high = mid;
        }
    }
    return low;
}
/**
 * `differenceWith` creates an array of values not included in the other arrays, using a custom comparator function.
 */
function differenceWith(array, comparator) {
    var arrays = [];
    for (var _i = 2; _i < arguments.length; _i++) {
        arrays[_i - 2] = arguments[_i];
    }
    var otherValues = new Set(arrays.flat());
    return array.filter(function (value) {
        return !otherValues.has(value) ||
            arrays.some(function (arr) {
                return arr.some(function (otherValue) { return comparator(value, otherValue); });
            });
    });
}
/**
 * Creates a slice of `array` with `n` elements dropped from the end.
 */
function dropRight(array, n) {
    if (n === void 0) { n = 1; }
    if (n >= array.length)
        return [];
    return array.slice(0, array.length - n);
}
/**
 * Creates a slice of `array` excluding elements dropped from the end until the predicate returns falsy.
 */
function dropRightWhile(array, predicate) {
    var dropIndex = array.length;
    for (var i = array.length - 1; i >= 0; i--) {
        if (!predicate(array[i], i, array)) {
            dropIndex = i + 1;
            break;
        }
    }
    return array.slice(0, dropIndex);
}
/**
 * Creates a slice of `array` excluding elements dropped from the beginning until the predicate returns falsy.
 */
function dropWhile(array, predicate) {
    var dropIndex = 0;
    for (var i = 0; i < array.length; i++) {
        if (!predicate(array[i], i, array)) {
            dropIndex = i;
            break;
        }
    }
    return array.slice(dropIndex);
}
/**
 * Creates an array of unique values that are included in all given arrays, using a custom iteratee function.
 */
function intersectionBy(arrays, iteratee) {
    var e_1, _a;
    if (arrays.length === 0)
        return [];
    var firstArray = arrays[0];
    var intersectedValues = [];
    var _loop_1 = function (value) {
        var computed = iteratee(value);
        if (intersectedValues.includes(computed))
            return "continue";
        var includesInAll = arrays
            .slice(1)
            .every(function (array) {
            return array.some(function (item) { return iteratee(item) === computed; });
        });
        if (includesInAll) {
            intersectedValues.push(computed);
        }
    };
    try {
        for (var firstArray_1 = __values(firstArray), firstArray_1_1 = firstArray_1.next(); !firstArray_1_1.done; firstArray_1_1 = firstArray_1.next()) {
            var value = firstArray_1_1.value;
            _loop_1(value);
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (firstArray_1_1 && !firstArray_1_1.done && (_a = firstArray_1.return)) _a.call(firstArray_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return intersectedValues.flatMap(function (computed) {
        return firstArray.filter(function (item) { return iteratee(item) === computed; });
    });
}
/**
 * Creates an array of unique values that are included in all given arrays, using a custom comparator function.
 */
function intersectionWith(arrays, comparator) {
    var e_2, _a;
    if (arrays.length === 0)
        return [];
    var firstArray = arrays[0];
    var intersectedValues = [];
    var _loop_2 = function (value) {
        if (intersectedValues.includes(value))
            return "continue";
        var includesInAll = arrays
            .slice(1)
            .every(function (array) { return array.some(function (item) { return comparator(item, value); }); });
        if (includesInAll) {
            intersectedValues.push(value);
        }
    };
    try {
        for (var firstArray_2 = __values(firstArray), firstArray_2_1 = firstArray_2.next(); !firstArray_2_1.done; firstArray_2_1 = firstArray_2.next()) {
            var value = firstArray_2_1.value;
            _loop_2(value);
        }
    }
    catch (e_2_1) { e_2 = { error: e_2_1 }; }
    finally {
        try {
            if (firstArray_2_1 && !firstArray_2_1.done && (_a = firstArray_2.return)) _a.call(firstArray_2);
        }
        finally { if (e_2) throw e_2.error; }
    }
    return intersectedValues;
}
/**
 * Removes all elements from `array` that match the comparator function.
 */
function pullAllWith(array, values, comparator) {
    return array.filter(function (item) { return !values.some(function (value) { return comparator(item, value); }); });
}
/**
 * Uses binary search to find the index of the last occurrence of a value in a sorted array.
 * If the value is not found, it returns -1.
 */
function sortedLastIndexOf(array, value) {
    var low = 0;
    var high = array.length - 1;
    var lastIndex = -1;
    while (low <= high) {
        var mid = Math.floor((low + high) / 2);
        var midValue = array[mid];
        if (midValue === value) {
            lastIndex = mid;
            low = mid + 1;
        }
        else if (midValue < value) {
            low = mid + 1;
        }
        else {
            high = mid - 1;
        }
    }
    return lastIndex;
}
/**
 * Creates a new array of unique values from the provided array,
 * using a custom iteratee function to generate the comparison values.
 * The input array must be sorted.
 */
function sortedUniqBy(array, iteratee) {
    var e_3, _a;
    var result = [];
    var previousValue = undefined;
    try {
        for (var array_1 = __values(array), array_1_1 = array_1.next(); !array_1_1.done; array_1_1 = array_1.next()) {
            var value = array_1_1.value;
            var computed = iteratee(value);
            if (computed !== previousValue) {
                result.push(value);
                previousValue = computed;
            }
        }
    }
    catch (e_3_1) { e_3 = { error: e_3_1 }; }
    finally {
        try {
            if (array_1_1 && !array_1_1.done && (_a = array_1.return)) _a.call(array_1);
        }
        finally { if (e_3) throw e_3.error; }
    }
    return result;
}
/**
 * Creates an array of unique values that is the union of all given arrays,
 * using a custom comparator function to determine equality.
 */
function unionWith(arrays, comparator) {
    var _a, e_4, _b;
    var result = [];
    var merged = (_a = []).concat.apply(_a, __spreadArray([], __read(arrays), false));
    var _loop_3 = function (item) {
        if (!result.some(function (existingItem) { return comparator(existingItem, item); })) {
            result.push(item);
        }
    };
    try {
        for (var merged_1 = __values(merged), merged_1_1 = merged_1.next(); !merged_1_1.done; merged_1_1 = merged_1.next()) {
            var item = merged_1_1.value;
            _loop_3(item);
        }
    }
    catch (e_4_1) { e_4 = { error: e_4_1 }; }
    finally {
        try {
            if (merged_1_1 && !merged_1_1.done && (_b = merged_1.return)) _b.call(merged_1);
        }
        finally { if (e_4) throw e_4.error; }
    }
    return result;
}
/**
 * Creates a new array of unique values from the provided array,
 * using a custom comparator function to determine equality.
 */
function uniqWith(array, comparator) {
    var e_5, _a;
    var result = [];
    var _loop_4 = function (item) {
        if (!result.some(function (existingItem) { return comparator(existingItem, item); })) {
            result.push(item);
        }
    };
    try {
        for (var array_2 = __values(array), array_2_1 = array_2.next(); !array_2_1.done; array_2_1 = array_2.next()) {
            var item = array_2_1.value;
            _loop_4(item);
        }
    }
    catch (e_5_1) { e_5 = { error: e_5_1 }; }
    finally {
        try {
            if (array_2_1 && !array_2_1.done && (_a = array_2.return)) _a.call(array_2);
        }
        finally { if (e_5) throw e_5.error; }
    }
    return result;
}
/**
 * This function is the reverse of `zipWith`.
 * It groups the elements of each array based on the provided iteratee function
 * and returns an array of grouped elements.
 */
function unzipWith(arrays, iteratee) {
    var length = arrays[0].length;
    var result = [];
    var _loop_5 = function (i) {
        result.push(iteratee.apply(void 0, __spreadArray([], __read(arrays.map(function (array) { return array[i]; })), false)));
    };
    for (var i = 0; i < length; i++) {
        _loop_5(i);
    }
    return result;
}
/**
 * Creates an array of unique values that is the symmetric difference of the given arrays,
 * using a custom iteratee function to generate the comparison values.
 */
function xorBy(arrays, iteratee) {
    var _a, e_6, _b;
    var result = [];
    var flattened = (_a = []).concat.apply(_a, __spreadArray([], __read(arrays), false));
    var seen = new Set();
    try {
        for (var flattened_1 = __values(flattened), flattened_1_1 = flattened_1.next(); !flattened_1_1.done; flattened_1_1 = flattened_1.next()) {
            var value = flattened_1_1.value;
            var computed = iteratee(value);
            if (!seen.has(computed)) {
                seen.add(computed);
                result.push(value);
            }
        }
    }
    catch (e_6_1) { e_6 = { error: e_6_1 }; }
    finally {
        try {
            if (flattened_1_1 && !flattened_1_1.done && (_b = flattened_1.return)) _b.call(flattened_1);
        }
        finally { if (e_6) throw e_6.error; }
    }
    return result;
}
/**
 * Creates an object composed of keys generated from the results of running each element of the array through the iteratee function.
 * The order of grouped values is determined by the order they occur in the array.
 */
function groupBy(array, iteratee) {
    return array.reduce(function (result, value) {
        var key = iteratee(value);
        if (!(key in result)) {
            result[key] = [];
        }
        result[key].push(value);
        return result;
    }, {});
}
/**
 * Checks if a given value is included in the array using strict equality for comparisons.
 */
function includes(array, value) {
    return array.includes(value);
}
/**
 * Invokes the method at `path` of each element in the array.
 */
function invokeMap(array, path) {
    return array.map(function (item) {
        var method = item[path.join("")];
        return typeof method === "function" ? method.call(item) : undefined;
    });
}
/**
 * Creates an object composed of keys generated from the results of running each element of the array through the iteratee function.
 */
function keyBy(array, iteratee) {
    return array.reduce(function (result, value) {
        var key = iteratee(value);
        result[key] = value;
        return result;
    }, {});
}
/**
 * Creates an array of values by running each element in the array through the iteratee function.
 */
function map(array, iteratee) {
    return array.map(iteratee);
}
/**
 * Creates an array of elements from the original array, sorted in ascending order by the results of running each element through the iteratees.
 */
function orderBy(array, iteratees, orders) {
    var mappedArray = array.map(function (item, index) { return ({ index: index, value: item }); });
    var comparison = function (a, b) {
        for (var i = 0; i < iteratees.length; i++) {
            var iteratee = iteratees[i];
            var aValue = iteratee(a.value);
            var bValue = iteratee(b.value);
            var order = orders && orders[i] === "desc" ? -1 : 1;
            if (aValue < bValue) {
                return -1 * order;
            }
            if (aValue > bValue) {
                return 1 * order;
            }
        }
        return a.index - b.index;
    };
    return mappedArray.sort(comparison).map(function (item) { return item.value; });
}
/**
 * Creates an array of elements split into two groups, the first of which contains elements predicate returns truthy for, while the second contains elements predicate returns falsy for.
 */
function partition(array, predicate) {
    return array.reduce(function (_a, value) {
        var _b = __read(_a, 2), truthy = _b[0], falsy = _b[1];
        if (predicate(value)) {
            truthy.push(value);
        }
        else {
            falsy.push(value);
        }
        return [truthy, falsy];
    }, [[], []]);
}
/**
 * Reduces the array to a single value by iterating over it from left to right.
 */
function reduce(array, iteratee, initialValue) {
    return array.reduce(iteratee, initialValue);
}
/**
 * Reduces the array to a single value by iterating over it from right to left.
 */
function reduceRight(array, iteratee, initialValue) {
    return array.reduceRight(iteratee, initialValue);
}
/**
 * Creates an array of values from the original array that do not satisfy the predicate.
 */
function reject(array, predicate) {
    return array.filter(function (value) { return !predicate(value); });
}
/**
 * Gets a random element from the array.
 */
function sample(array) {
    var randomIndex = Math.floor(Math.random() * array.length);
    return array[randomIndex];
}
/**
 * Creates an array of random elements from the original array up to the size limit.
 */
function sampleSize(array, size) {
    var shuffled = array.slice().sort(function () { return Math.random() - 0.5; });
    return shuffled.slice(0, Math.min(size, array.length));
}
/**
 * Creates a shuffled copy of the array.
 */
function shuffle(array) {
    return array.slice().sort(function () { return Math.random() - 0.5; });
}
/**
 * Returns the number of elements in the array.
 */
function size(array) {
    return array.length;
}
/**
 * Checks if any element in the array satisfies the predicate.
 */
function some(array, predicate) {
    return array.some(predicate);
}
/**
 * Creates a new array of sorted elements by the results of running each element of the original array through the iteratees.
 */
function sortBy(array, iteratees) {
    var mappedArray = array.map(function (item, index) { return ({ index: index, value: item }); });
    var comparison = function (a, b) {
        for (var i = 0; i < iteratees.length; i++) {
            var iteratee = iteratees[i];
            var aValue = iteratee(a.value);
            var bValue = iteratee(b.value);
            if (aValue < bValue) {
                return -1;
            }
            if (aValue > bValue) {
                return 1;
            }
        }
        return a.index - b.index;
    };
    return mappedArray.sort(comparison).map(function (item) { return item.value; });
}
/**
 * Creates an object composed of keys generated from the results of running each element of the array through the iteratee function.
 * The corresponding value of each key is an array of the elements responsible for generating the key.
 */
function countBy(array, iteratee) {
    return array.reduce(function (result, value) {
        var key = iteratee(value);
        if (!(key in result)) {
            result[key] = 0;
        }
        result[key]++;
        return result;
    }, {});
}
/**
 * Checks if every element in the array satisfies the predicate.
 */
function every(array, predicate) {
    return array.every(predicate);
}
/**
 * Creates a new array with all elements that pass the predicate test.
 */
function filter(array, predicate) {
    return array.filter(predicate);
}
/**
 * Returns the first element in the array that satisfies the predicate.
 */
function find(array, predicate) {
    return array.find(predicate);
}
/**
 * Returns the last element in the array that satisfies the predicate.
 */
function findLast(array, predicate) {
    return array.slice().reverse().find(predicate);
}
/**
 * Creates a new array by running each element in the array through the iteratee function and flattening the mapped results.
 * The iteratee function should return an array or iterable.
 */
function flatMap(array, iteratee) {
    return array.flatMap(iteratee);
}
/**
 * Recursively flattens the array up to the specified depth.
 */
function flatMapDeep(array, depth) {
    if (depth === void 0) { depth = 1; }
    if (depth <= 0) {
        return array.slice();
    }
    return array.flatMap(function (value) {
        return Array.isArray(value) ? flatMapDeep(value, depth - 1) : value;
    });
}
/**
 * Recursively flattens the array until the predicate function returns false.
 */
function flatMapDepth(array, predicate, depth) {
    if (depth === void 0) { depth = 1; }
    if (depth <= 0) {
        return array.slice();
    }
    return array.flatMap(function (value) {
        return predicate(value) ? flatMapDepth([value], predicate, depth - 1) : value;
    });
}
/**
 * Executes the provided callback function for each element in the array.
 */
function forEach(array, callback) {
    array.forEach(callback);
}
/**
 * Executes the provided callback function for each element in the array in reverse order.
 */
function forEachReverse(array, callback) {
    for (var i = array.length - 1; i >= 0; i--) {
        callback(array[i], i, array);
    }
}
/**
 * Executes the provided callback function for each element in the array from right to left.
 */
function forEachRight(array, callback) {
    array.slice().reverse().forEach(callback);
}
/**
 * Computes the Cartesian product of multiple sets.
 * The Cartesian product of sets A, B, C, ..., is the set of all ordered pairs (a, b, c, ...),
 * where a is in A, b is in B, c is in C, etc.
 *
 * This function takes an arbitrary number of arrays as input and
 * returns an array of arrays, each of which represents one possible
 * combination of one element from each input array.
 *
 * @param args An arbitrary number of arrays.
 * @returns The Cartesian product of the input arrays.
 */
function cartesian() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    var result = [], max = args.length - 1;
    function makeCartesian(arr, i) {
        for (var j = 0, l = args[i].length; j < l; j++) {
            var newArr = arr.slice(0);
            newArr.push(args[i][j]);
            if (i == max)
                result.push(newArr);
            else
                makeCartesian(newArr, i + 1);
        }
    }
    makeCartesian([], 0);
    return result;
}
/**
 * Returns the elements present in the first array but not in the second array.
 * @param firstArray - The first array.
 * @param secondArray - The second array.
 * @returns An array of elements unique to the first array.
 */
function getUniqueElementsInFirstArray(firstArray, secondArray) {
    var difference = firstArray.filter(function (element) { return !secondArray.includes(element); });
    return difference;
}
/**
 * Returns the elements that are unique to either of the two input arrays.
 * @param firstArray - The first array.
 * @param secondArray - The second array.
 * @returns An array of elements unique to either the first or the second array.
 */
function getUniqueElementsInEitherArray(firstArray, secondArray) {
    var difference = firstArray
        .filter(function (element) { return !secondArray.includes(element); })
        .concat(secondArray.filter(function (element) { return !firstArray.includes(element); }));
    return difference;
}
/**
 * Checks if all elements in the first array are also present in the second array.
 * @param firstArray - The first array.
 * @param secondArray - The second array.
 * @returns A boolean indicating whether the first array is a subset of the second.
 */
function isFirstArraySubsetOfSecond(firstArray, secondArray) {
    var result = firstArray.every(function (element) { return secondArray.includes(element); });
    return result;
}
/**
 * Returns all common elements between the two arrays.
 * @param firstArray - The first array.
 * @param secondArray - The second array.
 * @returns An array of elements common to both arrays.
 */
function getCommonElements(firstArray, secondArray) {
    var values = firstArray
        .filter(function (element) { return secondArray.includes(element); })
        .concat(secondArray.filter(function (element) { return firstArray.includes(element); }));
    var uniqueValues = __spreadArray([], __read(new Set(values)), false);
    return uniqueValues;
}
/**
 * Flattens an array of items. This function can also handle items that have child items.
 * Child items are retrieved using the provided `getChildren` function.
 * The resulting array contains all items and their children in a flat structure.
 * @param inputItems - An array of items to be flattened.
 * @param getChildren - A function that, given an item, returns its children or undefined.
 * @returns A flat array containing all input items and their children.
 */
function flattenBy(inputItems, getChildren) {
    var e_7, _a;
    var itemsToFlatten = new (Array.bind.apply(Array, __spreadArray([void 0], __read(inputItems), false)))();
    var result = [];
    while (itemsToFlatten.length > 0) {
        var item = itemsToFlatten.shift();
        if (!item)
            continue;
        result.push(item);
        var children = getChildren === null || getChildren === void 0 ? void 0 : getChildren(item);
        if (children) {
            try {
                for (var children_1 = (e_7 = void 0, __values(children)), children_1_1 = children_1.next(); !children_1_1.done; children_1_1 = children_1.next()) {
                    var child = children_1_1.value;
                    if (child) {
                        itemsToFlatten.push(child);
                    }
                }
            }
            catch (e_7_1) { e_7 = { error: e_7_1 }; }
            finally {
                try {
                    if (children_1_1 && !children_1_1.done && (_a = children_1.return)) _a.call(children_1);
                }
                finally { if (e_7) throw e_7.error; }
            }
        }
    }
    return result;
}
/**
 * Plucks the specified property from each object in the array.
 *
 * @template T, K
 * @param {T[]} array - The array of objects.
 * @param {K} key - The property to pluck from each object.
 * @returns {Array<K>} An array of the specified property values.
 */
function pluck(array, key) {
    return array.map(function (item) { return item[key]; });
}
/**
 * Retrieves the element at the specified index from an array.
 *
 * @template T - The type of elements in the array.
 * @param {T[]} array - The array from which to retrieve the element.
 * @param {number} index - The index of the element to retrieve. If negative, it is relative to the end of the array.
 * @returns {T | undefined} The element at the specified index, or undefined if the index is out of range.
 */
function getElementAt(array, index) {
    if (index < 0) {
        index = array.length + index;
    }
    return array[index];
}
function inArray(element, array) {
    return array.indexOf(element);
}
function grep(array, callback) {
    return array.filter(callback);
}
export { cartesian, chunk, compact, concat, countBy, difference, differenceBy, differenceWith, drop, dropRight, dropRightWhile, dropWhile, every, fill, filter, find, findIndex, findLast, findLastIndex, first, flatMap, flatMapDeep, flatMapDepth, flatten, flattenBy, flattenDeep, flattenDepth, forEach, forEachReverse, forEachRight, fromPairs, getCommonElements, getElementAt, getUniqueElementsInEitherArray, getUniqueElementsInFirstArray, grep, groupBy, head, inArray, includes, indexOf, initial, intersection, intersectionBy, intersectionWith, invokeMap, isFirstArraySubsetOfSecond, join, keyBy, last, lastIndexOf, map, nth, orderBy, partition, pluck, pull, pullAll, pullAllBy, pullAllWith, pullAt, reduce, reduceRight, reject, remove, reverse, sample, sampleSize, shuffle, size, slice, some, sortBy, sortedIndex, sortedIndexBy, sortedIndexOf, sortedLastIndex, sortedLastIndexOf, sortedUniq, sortedUniqBy, tail, take, takeRight, takeRightWhile, takeWhile, union, unionBy, unionWith, uniq, uniqBy, uniqWith, unzip, unzipWith, without, xor, xorBy, xorWith, zip, zipObject, zipObjectDeep, zipWith, };
//# sourceMappingURL=array-utilities.js.map