declare function now(): number;
declare function getISO8601Now(): string;
declare function getFormattedDate(date: Date | string, separator?: string): string;
/**
 * Formats a date to a string using the specified format.
 * @param date - The date object to format.
 * @param format - The format string (e.g., 'yyyy-MM-dd HH:mm:ss').
 * @returns The locals string or string[].
 */
declare function formatDate(date: Date, locals: string | string[]): string;
/**
 * Gets the current date and time.
 * @returns The current date object.
 */
declare function getCurrentDate(): Date;
/**
 * Calculates the difference in days between two dates.
 * @param date1 - The first date.
 * @param date2 - The second date.
 * @returns The difference in days.
 */
declare function getDateDiffInDays(date1: Date, date2: Date): number;
/**
 * Adds or subtracts a specified number of days to/from a date.
 * @param date - The date to modify.
 * @param days - The number of days to add or subtract (negative for subtraction).
 * @returns The modified date object.
 */
declare function addDays(date: Date, days: number): Date;
/**
 * Checks if a year is a leap year.
 * @param year - The year to check.
 * @returns `true` if the year is a leap year, otherwise `false`.
 */
declare function isLeapYear(year: number): boolean;
/**
 * Gets the first day of a specified month.
 * @param year - The year of the month.
 * @param month - The month (0-based index).
 * @returns The first day of the month.
 */
declare function getFirstDayOfMonth(year: number, month: number): Date;
/**
 * Checks if two dates are equal.
 * @param date1 - The first date.
 * @param date2 - The second date.
 * @returns `true` if the dates are equal, otherwise `false`.
 */
declare function areDatesEqual(date1: Date, date2: Date): boolean;
/**
 * Gets the last day of a specified month.
 * @param year - The year of the month.
 * @param month - The month (0-based index).
 * @returns The last day of the month.
 */
declare function getLastDayOfMonth(year: number, month: number): Date;
/**
 * Checks if a date is in the future.
 * @param date - The date to check.
 * @returns `true` if the date is in the future, otherwise `false`.
 */
declare function isDateInFuture(date: Date): boolean;
/**
 * Checks if a date is within a specified range.
 * @param date - The date to check.
 * @param startDate - The start date of the range.
 * @param endDate - The end date of the range.
 * @returns `true` if the date is within the range, otherwise `false`.
 */
declare function isDateInRange(date: Date, startDate: Date, endDate: Date): boolean;
/**
 * Calculates the age based on the provided birth date.
 * @param birthDate - The birth date.
 * @returns The age in years.
 */
declare function calculateAge(birthDate: Date): number;
/**
 * Checks if a given year is a future year.
 * @param year - The year to check.
 * @returns `true` if the year is in the future, otherwise `false`.
 */
declare function isFutureYear(year: number): boolean;
/**
 * Checks if a date is a weekend day (Saturday or Sunday).
 * @param date - The date to check.
 * @returns `true` if the date is a weekend day, otherwise `false`.
 */
declare function isWeekend(date: Date): boolean;
/**
 * Converts a date string in ISO 8601 format to a Date object.
 * @param dateString - The date string in ISO 8601 format.
 * @returns The parsed Date object, or `null` if the date string is invalid.
 */
declare function parseISODate(dateString: string): Date | null;
export { addDays, areDatesEqual, calculateAge, formatDate, getCurrentDate, getDateDiffInDays, getFirstDayOfMonth, getFormattedDate, getISO8601Now, getLastDayOfMonth, isDateInFuture, isDateInRange, isFutureYear, isLeapYear, isWeekend, now, parseISODate, };
//# sourceMappingURL=date-utilities.d.ts.map